import 'package:bokain_models/src/model_base.dart';

class Day extends ModelBase
{
  static const int startHour = 7;
  static const int startMinute = 0;
  static const int endHour = 19;
  static const int endMinute = 0;

  Day(this._startTime) : super()
  {
    _startTime = new DateTime(
        _startTime.year,
        _startTime.month,
        _startTime.day,
        startHour,
        startMinute);

    _endTime = new DateTime(
        _startTime.year,
        _startTime.month,
        _startTime.day,
        endHour,
        endMinute);

    increments = new List();
    DateTime iTime = _startTime;
    while (iTime.isBefore(_endTime))
    {
      increments.add(new Increment(iTime));
      iTime = iTime.add(Increment.duration);
    }
  }

  @override
  void parseData(Map<String, dynamic> data)
  {
    userId = data["userId"];

    Iterable<Map<String, String>> activeIncrements = data["activeIncrements"];
    for (Map<String, dynamic> incrementData in activeIncrements)
    {
      DateTime startTime = DateTime.parse(incrementData["startTime"]);
      Increment i = increments.firstWhere((i) => i.startTime.isAtSameMomentAs(startTime));
      i.open = incrementData["open"] as bool;
      i.bookingId = incrementData["bookingId"];
    }
  }

  @override
  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["userId"] = userId;

    Iterable<Increment> activeIncrements = increments.where((i) => i.open || i.bookingId != null);
    data["activeIncrements"] = activeIncrements.map((i) => i.properties);
    return data;
  }

  DateTime _startTime, _endTime;
  String userId;
  List<Increment> increments;
}

class Increment
{
  static const Duration duration = const Duration(minutes: 10);

  Increment(this._startTime)
  {
    _endTime = _startTime.add(duration);
  }

  Map<String, dynamic> get properties
  {
    Map<String, dynamic> data = new Map();
    data["startTime"] = _startTime.toIso8601String();
    data["endTime"] = _endTime.toIso8601String();
    data["bookingId"] = bookingId;
    data["open"] = open;
    return data;
  }

  DateTime get startTime => _startTime;
  DateTime get endTime => _endTime;

  final DateTime _startTime;
  DateTime _endTime;

  bool open = false;
  String bookingId;
}